#!/bin/bash

USEADMIN=""

die() {
        echo "== ERROR: $*" >&2
        exit 1
}

#[ `whoami` == 'zimbra' ] || die "not started as zimbra user"

die_help() {
        if [ "$*" ]; then
                echo "$*" >&2
                echo "" >&2
        fi
        echo "Synopis:" >&2
        echo "mandatory:" >&2
        echo "$0 --users <userlistfile> --config <configfile> " >&2
        echo "optional:" >&2
        echo "$0 --admin	use admin AUTH, admin prefix from config, prompt for password" >&2
        exit 1
}

get_local_zimbraserver() {
  ZMHOSTNAME=`which zmhostname`
  [ "$ZMHOSTNAME" ] || die "zmhostname not found in path"
  ZIMBRASERVER=`$ZMHOSTNAME`
}

check_params() {

  	ZMPROV=`which zmprov`
	[ "$ZMPROV" ]		|| die "zmprov not found in path - are you sure you are running on a zimbra system?"

        [ "$CONFIGFILE" ]    	|| die_help "--config missing"
        [ -f "$CONFIGFILE" ] 	|| die_help "config file not found: $CONFIGFILE"

. $CONFIGFILE || die "failed to load $PROVISIONER_CONFIG"

	[ "$IMAPSYNC" ]	     	|| IMAPSYNC=`which imapsync`
	[ "$IMAPSYNC" ]	     	|| die "imapsync neither found in path nor in config"
	[ "$ZMIGRATIONDOMAIN" ]	|| die "missing setting for migration domain"
	[ -f "$IMAPSYNC" ]   	|| die "$IMAPSYNC not found"
	[ "$USEADMIN" ]      	|| USEADMIN="no"
	[ "$EXCHANGESERVER" ]	|| die "Exchange server not set in config"
	[ "$ZIMBRAPSERVER" ] 	|| get_local_zimbraserver
#	[ "$ZIMBRAIMAPPORT" ]	|| die "missing setting for zimbra IMAP port"
#	[ "$ZIMBRASSL" ]	|| die "missing setting for zimbra IMAP SSL mode"

        if [ "$USEADMIN" = "yes" ]; then
          [ "$ADMINPREFIX" ]      || die "missing Exchange admin prefix"
          read -s -p "Exchange Admin Password: " ADMINPW
          [ "$ADMINPW" ]          || die "Exchange admin password should not be empty..."

	fi
	[ "$USERLISTFILE" ]	|| die_help "--users missing"
	[ -f "$USERLISTFILE" ]	|| die_help "$USERLISTFILE not found"

	[ "$TMPDIR" ]		|| TMPDIR="/tmp" 
}

## parse command line arguments
while [ "$1" ]; do
        case "$1" in
                --config)
                        shift
                        CONFIGFILE="$1"
                        shift
                ;;
                --users)
                        shift
                        USERLISTFILE="$1"
                        shift
                ;;
                --admin)
                        shift
                        USEADMIN="yes"
                        shift
                ;;
                --help)
                        die_help
                ;;
                *)
                        die_help "unsupported argument: $1"
                ;;
        esac
done

do_imap_sync() {
	for EXUSER in `cat $USERLISTFILE | grep -v "^#" | awk -F ";" '{print $1}'`; do
		EXPW=`cat $USERLISTFILE | grep -v "^#" | grep "^$EXUSER" | awk -F ";" '{print $2}'`
		ZUSER=`cat $USERLISTFILE | grep -v "^#" | grep "^$EXUSER" | awk -F ";" '{print $3}'`
		[ "$ZUSER" ] || ZUSER=$EXUSER
		EGS=`( echo "a" ; $ZMMAILBOX -z -m $ZUSER gaf| \
                        awk '{print $1 ";" $2}' | egrep "(appo|task|cont|docu)" | \
                        awk -F ";" '{print "^"$1 "|" }' ; echo "n") | tr -d '\n' | sed 's/|n/)/' | sed 's/a/(/'`
  		
		EXCLUDES=`($ZMMAILBOX -z -m $ZUSER gaf | sed -e 's/^[ \t]*//' | egrep "$EGS" | \
			awk -F " /" '{print $2}' | awk -F "(" '{print $1 "|"}'; echo "_x_x_x_") | \
			tr -d '\n' | sed 's/|_x_x_x_//' `
                ZLOCALPART=`echo "$ZUSER" | awk -F "@" '{print $1}'`;
                ZMUSER=`echo "$ZLOCALPART@$ZMIGRATIONDOMAIN"`
		zmprov ra $ZUSER $ZMUSER
		zmprov sp $ZMUSER 'TempPass4Migration'		
		if [ "$USEADMIN" = "yes" ]; then
		/usr/bin/perl $IMAPSYNC	--host1 $EXCHANGESERVER -tls1 --user1 "$ADMINPREFIX\\$EXUSER" --password1 "$ADMINPW" \
			--host2 $ZIMBRAPSERVER -tls2 --user2 $ZUSER --password2 "TempPass4Migration" \
			--regextrans2 's/(.)*Gesendete(.)*$/Sent/' \
			--regextrans2 's/(.)*Postausgang(.)*$/Sent/' \
			--regextrans2 's/(.)*Sent Items(.)*$/Sent/' \
			--regextrans2 's/(.)*Gel&APY-schte(.)*$/Trash/' \
			--regextrans2 's/(.)*Deleted(.)*$/Trash/' \
			--regextrans2 's/(.)*Entw&APw-rfe(.)*$/Drafts/' \
			--authmech2 PLAIN --skipsize --exclude "Kalender|Aufgaben|Kontakte|Notizen|Notes|$EXCLUDES" 2>&1 | logger -t "migration"
		else
		echo "/usr/bin/perl $IMAPSYNC --host1 $EXCHANGESERVER -tls1 --user1 \"$EXUSER\" --password1 \"$ADMINPW\" --host2 $ZIMBRAPSERVER -tls2 --user2 $ZUSER --password2 \"$ZPW\" --authmech2 PLAIN --skipsize --exclude \"Kalender|Aufgaben|Kontakte|$EXCLUDES\""
  		fi
			zmprov ra $ZMUSER $ZUSER
		done
}

check_params
do_imap_sync
